Scriptname vibBedAliasSCT extends ReferenceAlias Hidden

vibMUTEXSCT Property mutex Auto
String Property mutexName Auto

Actor Property PlayerREF Auto
Quest Property vibSleepQST Auto
Quest Property vibSleepNPCsQST Auto
vibMainSCT Property vibMainQST Auto
Faction Property PlayerBedOwnership Auto
ReferenceAlias Property SpouseAlias Auto
GlobalVariable Property vibPlayerSLeepState Auto

ImageSpaceModifier Property vibFadeToBlackImod Auto
ImageSpaceModifier Property vibFadeToBlackHoldImod Auto
ImageSpaceModifier Property vibFadeToBlackBackImod Auto

Event OnInit()
	GoToState("")
	mutex.lock(mutexName)
	If (vibMainQST.Uninstall)
		mutex.unlock(mutexName)
		Return
	EndIf

	Int iterations = 0
	While (!GetReference() && iterations < 3)
		Utility.Wait(0.2)
		iterations += 1
	EndWhile

	If (vibMainQST.BedAliasBuffer && vibMainQST.BedAliasBuffer != GetReference())
		vibMainQST.BedAliasBuffer.BlockActivation(False)
	EndIf

	If (GetReference())
		ActorBase BedOwnerAct = GetReference().GetActorOwner()
		Faction BedOwnerFact = GetReference().GetFactionOwner()

		If (\
				(!BedOwnerAct || BedOwnerAct == PlayerREF.GetActorBase() || BedOwnerAct.GetDeadCount() > 0) &&\
				(!BedOwnerFact || PlayerREF.IsInFaction(BedOwnerFact))\
			)
			GoToState("ActivationBlocked")

			GetReference().BlockActivation(True)
			vibMainQST.BedAliasBuffer = GetReference()
		Else
			vibMainQST.BedAliasBuffer = None
		EndIf
	EndIf

	mutex.unlock(mutexName)
EndEvent

State ActivationBlocked
	Event OnActivate(ObjectReference akActionRef)
		If (!mutex.lock(mutexName))
			If (akActionRef == PlayerREF)
				GoToState("GoingToBed")
				vibPlayerSLeepState.SetValue(0)
				vibSleepNPCsQST.Stop()
				vibSleepQST.Stop()
				vibSleepQST.Start()
				vibSleepNPCsQST.Start()
				Game.ForceThirdPerson()
				Game.DisablePlayerControls(True, True, True, False, True, True, True, True, 0)
				Game.SetPlayerAIDriven(True)
				RegisterForSingleUpdate(0.5)
				vibMainQST.Undress()

			Else
				mutex.unlock(mutexName)
			EndIf
		Else
			mutex.unlock(mutexName)
		EndIf
	EndEvent

EndState

Function HaltAnimation()
	GoToState("")
	vibFadeToBlackBackImod.Remove()
	vibFadeToBlackHoldImod.Remove()
	vibFadeToBlackImod.Remove()

	Game.SetPlayerAIDriven(False)
	Game.EnablePlayerControls()
	vibSleepQST.Stop()
	PlayerREF.SetScale(vibMainQST.PlayerScale)

	If (vibMainQST.SpouseEnabled)
		If (SpouseAlias.GetActorReference())
			(SpouseAlias As vibSpouseAliasSCT).Dressup()
		EndIf

		mutex.unlock("SpouseAlias")
		mutex.lock("SpouseAlias")
		vibSleepNPCsQST.Stop()
		mutex.unlock("SpouseAlias")
	Else
		vibSleepNPCsQST.Stop()
	EndIf

	mutex.unlock(mutexName)
EndFunction

State GoingToBed

	Event OnUpdate()
		If (PlayerREF.GetSleepState() < 3)
			RegisterForSingleUpdate(0.5)
		Else
			GoToState("Sleeping")
			If (StringUtil.Find("$Coffin", GetReference().GetBaseObject().GetName()) >= 0)
				Game.ForceFirstPerson()
			EndIf
			vibMainQST.PlayerScale = PlayerREF.GetScale()
			Game.DisablePlayerControls(True, True, True, False, True, True, True, True, 0)
			RegisterForAnimationEvent(PlayerREF, "IdleFurnitureExit")

			vibPlayerSLeepState.SetValue(3)
			If (vibMainQST.SpouseEnabled)
				If (SpouseAlias.GetActorReference() && SpouseAlias.GetActorReference().GetParentCell() == PlayerREF.GetParentCell())
					Utility.Wait(0.5)
					SpouseAlias.GetActorReference().EvaluatePackage()
					Int iterations = 0
					While (iterations < 256 &&\
						PlayerREF.GetSleepState() == 3 &&\
						SpouseAlias.GetActorReference().GetSleepState() < 3)
						Utility.Wait(0.2)
						iterations += 1
					EndWhile
				EndIf
			EndIf
			

			If (PlayerREF.GetSleepState() != 3)
				Return
			EndIf

			vibFadeToBlackImod.Apply()
			Utility.Wait(2.25)
			vibFadeToBlackHoldImod.ApplyCrossFade(0.5)
			Utility.Wait(0.5)
			PlayerREF.MoveTo(PlayerREF, 0, 0, 0, True)
			Utility.Wait(0.5)

			If (GetReference())
				GetReference().Activate(PlayerREF, True)
			Else
				HaltAnimation()
				Return
			EndIf
			Utility.Wait(0.5)

			Bool NormalProcess = False
			If (GetReference())
				NormalProcess = GetReference().GetParentCell() == PlayerREF.GetParentCell()
			Else
				HaltAnimation()
				Return
			EndIf

			If (NormalProcess && GetReference())
				PlayerREF.MoveTo(GetReference(), 0, 0, 0, True)
				Game.DisablePlayerControls(True, True, True, False, True, True, True, True, 0)
				Utility.Wait(0.5)
			EndIf
			PlayerREF.SetScale(vibMainQST.PlayerScale)

			vibFadeToBlackHoldImod.PopTo(vibFadeToBlackBackImod)
			Utility.Wait(2.0)

			vibFadeToBlackBackImod.Remove()
			vibFadeToBlackHoldImod.Remove()
			vibFadeToBlackImod.Remove()

			If (NormalProcess)
				If (vibMainQST.SnoozeDuration > 0)
					Utility.Wait(vibMainQST.SnoozeDuration)
					If (GetReference())
						GetReference().Activate(PlayerREF, True)
						If (StringUtil.Find("$Coffin", GetReference().GetBaseObject().GetName()) >= 0)
							Game.ForceThirdPerson()
						EndIf
						Game.SetPlayerAIDriven(False)
						Game.EnablePlayerControls(True, True, False, True, False, True, True, True, 0)
						vibSleepQST.Stop()
					EndIf
				Else
					Game.EnablePlayerControls(False, False, False, False, False, False, True, False, 0)
				EndIf
			Else
				GoToState("")
				Game.SetPlayerAIDriven(False)
				Game.EnablePlayerControls()
				vibSleepQST.Stop()
				
				Utility.Wait(vibMainQST.UpdateInterval)

				If (vibMainQST.SpouseEnabled)
					If (!SpouseAlias.GetActorReference())
						mutex.unlock("SpouseAlias")
					ElseIf (SpouseAlias.GetActorReference().GetSleepState() == 0 ||\
						SpouseAlias.GetActorReference().GetParentCell() != PlayerREF.GetParentCell())
						(SpouseAlias As vibSpouseAliasSCT).Dressup()
						mutex.unlock("SpouseAlias")
					EndIf
					mutex.lock("SpouseAlias")
					vibSleepNPCsQST.Stop()
					mutex.unlock("SpouseAlias")
				Else
					vibSleepNPCsQST.Stop()
				EndIf

				mutex.unlock(mutexName)
			EndIf

		EndIf
	EndEvent

EndState

State Sleeping

	Event OnAnimationEvent(ObjectReference akSource, string asEventName)
		GoToState("")
		Game.SetPlayerAIDriven(False)
		Game.EnablePlayerControls(True, True, False, True, False, True, True, True, 0)
		vibSleepQST.Stop()

		UnregisterForAnimationEvent(PlayerREF, "IdleFurnitureExit")
		Utility.Wait(vibMainQST.UpdateInterval)
		vibMainQST.RefreshFace()
		vibMainQST.Dressup()
		Game.EnablePlayerControls()

		If (vibMainQST.SpouseEnabled)
			If (SpouseAlias.GetActorReference())

				Int iterations = 0
				While (iterations < 128 &&\
					mutex.isLocked("SpouseAlias") &&\
					SpouseAlias.GetActorReference() &&\
					SpouseAlias.GetActorReference().GetSleepState() != 0 &&\
					SpouseAlias.GetActorReference().GetParentCell() == PlayerREF.GetParentCell())
					Utility.Wait(0.2)
					iterations += 1
				EndWhile

				Utility.Wait(vibMainQST.UpdateInterval)

				If (!SpouseAlias.GetActorReference())
					mutex.unlock("SpouseAlias")
				ElseIf (SpouseAlias.GetActorReference().GetSleepState() == 0 ||\
					SpouseAlias.GetActorReference().GetParentCell() != PlayerREF.GetParentCell())
					(SpouseAlias As vibSpouseAliasSCT).Dressup()
					mutex.unlock("SpouseAlias")
				EndIf

				mutex.lock("SpouseAlias")
				vibSleepNPCsQST.Stop()
				mutex.unlock("SpouseAlias")
			EndIf
		Else
			vibSleepNPCsQST.Stop()
		EndIf

		mutex.unlock(mutexName)
	EndEvent

EndState
