Scriptname vibMCMenuSCT extends SKI_ConfigBase Hidden

vibMainSCT Property vibMainQST Auto
GlobalVariable Property vibSpouseEnabled Auto
GlobalVariable Property vibFollowersEnabled Auto

Int Property Revision Auto Hidden
Int Property NewRevision Hidden
	Int Function Get()
		Return 1
	EndFunction
EndProperty

Int OptionCount
Int[] OptionIDs
String[] OptionNames
String[] FormatStrings

Function CheckRevision()
	If (Revision < NewRevision)
		OnRevisionUpdate(Revision, NewRevision)
		Revision = NewRevision
	EndIf
EndFunction

Int Function RegisterOption(String asOptionName, Int aiOptionID, String asFormatString = "{2}")
	OptionIDs[OptionCount] = aiOptionID
	OptionNames[OptionCount] = asOptionName
	FormatStrings[OptionCount] = asFormatString
	OptionCount += 1
	Return aiOptionID
EndFunction

Int Function FindOptionIDbyName(String asOptionName)
	Int OptionIndex = OptionNames.Find(asOptionName)
	If (OptionIndex >= 0)
		Return OptionIDs[OptionIndex]
	EndIf
	Return -1
EndFunction

Event OnRevisionUpdate(Int aiRevision, Int aiNewRevision)
	If (aiRevision < 1)
	EndIf

EndEvent

Event OnPageReset(String asPage)
	CheckRevision()

	OptionCount = 0
	OptionIDs = New Int[128]
	OptionNames = New String[128]
	FormatStrings = New String[128]

	SetCursorFillMode(TOP_TO_BOTTOM)

	SetCursorPosition(0)

	AddHeaderOption("$Settings")
	RegisterOption("Sleepwear", AddMenuOption("$Sleepwear", "$" + vibMainQST.UndressMethod))
	RegisterOption("Snooze Duration", AddSliderOption("$SnoozeDuration", vibMainQST.SnoozeDuration, "{0} sec"), "{0} sec")
	RegisterOption("Spouse Enabled", AddToggleOption("$SpouseEnabled", vibMainQST.SpouseEnabled))
	RegisterOption("Followers Enabled", AddToggleOption("$FollowersEnabled", vibMainQST.FollowersEnabled))

	AddEmptyOption()
	AddHeaderOption("$Maintenance")
	; RegisterOption("Update Interval", AddSliderOption("Update Interval", vibMainQST.UpdateInterval, "{1} sec"), "{1} sec")
	RegisterOption("Reset", AddToggleOption("$ResetControls", vibMainQST.ResetControls))
	RegisterOption("Uninstall", AddToggleOption("$Uninstall", vibMainQST.Uninstall))

	SetCursorPosition(1)
	AddHeaderOption("$DetectedFiles")
	Int index = 0
	While (index < vibMainQST.DLCisLoaded.Length)
		AddToggleOption(vibMainQST.DLCfileNames[index], vibMainQST.DLCisLoaded[index], OPTION_FLAG_DISABLED)
		index += 1
	EndWhile
EndEvent

Event OnOptionSelect(Int aiOptionID)
	Int OptionIndex = OptionIDs.Find(aiOptionID)
	String OptionName = OptionNames[OptionIndex]

	If (OptionName == "Spouse Enabled")
		Bool checked = !vibMainQST.SpouseEnabled
		vibMainQST.SpouseEnabled = checked
		vibSpouseEnabled.SetValue(vibMainQST.SpouseEnabled As Int)
		SetToggleOptionValue(aiOptionID, checked)

	ElseIf (OptionName == "Followers Enabled")
		Bool checked = !vibMainQST.FollowersEnabled
		vibMainQST.FollowersEnabled = checked
		vibSpouseEnabled.SetValue(vibMainQST.FollowersEnabled As Int)
		SetToggleOptionValue(aiOptionID, checked)

	ElseIf (OptionName == "Reset")
		Utility.WaitMenuMode(0.1)
		vibMainQST.ResetControls = True
		SetToggleOptionValue(aiOptionID, True)
		; vibMainQST.ResetControls()

	ElseIf (OptionName == "Uninstall")
		If (!vibMainQST.Uninstall)
			Utility.WaitMenuMode(0.5)
			String Msg = "$UNINSTALL_MSG"
			Bool continue = ShowMessage(Msg, True, "$Yes", "$No")
			If (continue)
				Utility.WaitMenuMode(0.1)
				vibMainQST.Uninstall = True
				SetToggleOptionValue(aiOptionID, vibMainQST.Uninstall)
				; vibMainQST.ResetControls()
			EndIf
		EndIf

	EndIf

EndEvent

Event OnOptionMenuOpen(Int aiOptionID)
	Int OptionIndex = OptionIDs.Find(aiOptionID)
	String OptionName = OptionNames[OptionIndex]

	If (OptionName == "Sleepwear")
		String[] UndressMethodValueSet = New String[4]
		UndressMethodValueSet[0] = "$" + vibMainQST.UndressMethodValueSet[0]
		UndressMethodValueSet[1] = "$" + vibMainQST.UndressMethodValueSet[1]
		UndressMethodValueSet[2] = "$" + vibMainQST.UndressMethodValueSet[2]
		UndressMethodValueSet[3] = "$" + vibMainQST.UndressMethodValueSet[3]
		SetMenuDialogOptions(UndressMethodValueSet)
		SetMenuDialogStartIndex(vibMainQST.UndressMethodValueSet.Find(vibMainQST.UndressMethod))
		SetMenuDialogDefaultIndex(vibMainQST.UndressMethodValueSet.Find(vibMainQST.UndressMethodDefault))
	EndIf

EndEvent

Bool UnequippedFirstTime = False
Event OnOptionMenuAccept(Int aiOptionID, Int aiIndex)
	Int OptionIndex = OptionIDs.Find(aiOptionID)
	String OptionName = OptionNames[OptionIndex]

	If (OptionName == "Sleepwear")
		If (vibMainQST.UndressMethodValueSet[aiIndex] != "SLP_DISABLED" && !UnequippedFirstTime)
			Utility.WaitMenuMode(0.5)
			String Msg = "$SLEEPWEAR_MSG"
			Bool continue = ShowMessage(Msg, True, "$Yes", "$No")
			If (continue)
				UnequippedFirstTime = True
				Game.GetPlayer().UnequipAll()
				vibMainQST.UndressMethod = vibMainQST.UndressMethodValueSet[aiIndex]
				SetMenuOptionValue(aiOptionID, "$" + vibMainQST.UndressMethod)
			EndIf
		Else
			vibMainQST.UndressMethod = vibMainQST.UndressMethodValueSet[aiIndex]
			SetMenuOptionValue(aiOptionID, "$" + vibMainQST.UndressMethod)
		EndIf
	EndIf
EndEvent

Event OnOptionSliderOpen(Int aiOptionID)
	Int OptionIndex = OptionIDs.Find(aiOptionID)
	String OptionName = OptionNames[OptionIndex]

	If (OptionName == "Snooze Duration")
		SetSliderDialogStartValue(vibMainQST.SnoozeDuration)
		SetSliderDialogDefaultValue(vibMainQST.SnoozeDurationDefault)
		SetSliderDialogRange(vibMainQST.SnoozeDurationMin, vibMainQST.SnoozeDurationMax)
		SetSliderDialogInterval(1)
	ElseIf (OptionName == "Update Interval")
		SetSliderDialogStartValue(vibMainQST.UpdateInterval)
		SetSliderDialogDefaultValue(vibMainQST.UpdateIntervalDefault)
		SetSliderDialogRange(0.1, 3.0)
		SetSliderDialogInterval(0.1)
	EndIf

EndEvent

Event OnOptionSliderAccept(Int aiOptionID, Float afValue)
	Int OptionIndex = OptionIDs.Find(aiOptionID)
	String OptionName = OptionNames[OptionIndex]

	If (OptionName == "Snooze Duration")
		vibMainQST.SnoozeDuration = afValue
	ElseIf (OptionName == "Update Interval")
		vibMainQST.UpdateInterval = afValue
	EndIf

	SetSliderOptionValue(aiOptionID, afValue, FormatStrings[OptionIndex])

EndEvent

Event OnOptionDefault(Int aiOptionID)
	Int OptionIndex = OptionIDs.Find(aiOptionID)
	String OptionName = OptionNames[OptionIndex]

	If (OptionName == "Sleepwear")
		vibMainQST.UndressMethod = vibMainQST.UndressMethodDefault
		SetMenuOptionValue(aiOptionID, "$" + vibMainQST.UndressMethod)
	ElseIf (OptionName == "Snooze Duration")
		vibMainQST.SnoozeDuration = vibMainQST.SnoozeDurationDefault
		SetSliderOptionValue(aiOptionID, vibMainQST.SnoozeDuration, FormatStrings[OptionIndex])
	ElseIf (OptionName == "Update Interval")
		vibMainQST.UpdateInterval = vibMainQST.UpdateIntervalDefault
		SetSliderOptionValue(aiOptionID, vibMainQST.UpdateInterval, FormatStrings[OptionIndex])
	ElseIf (OptionName == "Spouse Enabled")
		vibMainQST.SpouseEnabled = vibMainQST.SpouseEnabledDefault
		vibSpouseEnabled.SetValue(vibMainQST.SpouseEnabled As Int)
		SetToggleOptionValue(aiOptionID, vibMainQST.SpouseEnabled)
	ElseIf (OptionName == "Followers Enabled")
		vibMainQST.FollowersEnabled = vibMainQST.FollowersEnabledDefault
		vibFollowersEnabled.SetValue(vibMainQST.FollowersEnabled As Int)
		SetToggleOptionValue(aiOptionID, vibMainQST.FollowersEnabled)
	ElseIf (OptionName == "Reset")
	ElseIf (OptionName == "Uninstall")
	EndIf

EndEvent

Event OnOptionHighlight(Int aiOptionID)
	Int OptionIndex = OptionIDs.Find(aiOptionID)
	String OptionName = OptionNames[OptionIndex]

	If (OptionName == "Sleepwear")
		String Msg = "$HIGHLIGHT_SLEEPWEAR"
		If (vibMainQST.UndressMethod == "SLP_COMFORTABLE")
			Msg = "$HIGHLIGHT_COMFORTABLE"
		ElseIf (vibMainQST.UndressMethod == "SLP_CLOTHES")
			Msg = "$HIGHLIGHT_CLOTHES"
		ElseIf (vibMainQST.UndressMethod == "SLP_NUDE")
			Msg = "$HIGHLIGHT_NUDE"
		EndIf
		SetInfoText(Msg)
	ElseIf (OptionName == "Snooze Duration")
		SetInfoText("$HIGHLIGHT_SNOOZE")
	ElseIf (OptionName == "Update Interval")
		SetInfoText("$HIGHLIGHT_UPDATE")
	ElseIf (OptionName == "Spouse Enabled")
		SetInfoText("$HIGHLIGHT_SPOUSE")
	ElseIf (OptionName == "Followers Enabled")
		SetInfoText("$HIGHLIGHT_FOLLOW")
	ElseIf (OptionName == "Reset")
		SetInfoText("$HIGHLIGHT_RESET")
	ElseIf (OptionName == "Uninstall")
		SetInfoText("$HIGHLIGHT_UNINSTALL")
	EndIf

EndEvent